﻿using Dto;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Shared;

namespace API.Controllers
{
    [Route("api/[controller]")]
    [Authorize]
    [ApiController]
    public class InquiriesController()
        : ControllerBase
    {
        private readonly IInquiryService<InquiryDTO> _inquiryService;
        private readonly ILogger<InquiriesController> _logger;

        public InquiriesController(IInquiryService<InquiryDTO> inquiryService, ILogger<InquiriesController> logger) : this()
        {
            _inquiryService = inquiryService;
            _logger = logger;
        }

        [HttpGet("inquiries/{page}/{number}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 204)]
        public IActionResult GetInquiries(int page, int number)
        {
            var nbInquiry = _inquiryService.GetInquiries(page, number).Count();
            if (nbInquiry == 0)
            {
                _logger.LogError("[ERREUR] Aucune enquête trouvé.");
                return StatusCode(204);
            }

            _logger.LogInformation("[INFORMATION] {nb} Enquête(s) trouvée(s)", nbInquiry);
            return Ok(_inquiryService.GetInquiries(page, number));
        }

        [HttpGet("inquiry/id/{id}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 404)]
        public IActionResult GetInquiryById(int id)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] Enquête avec l'id {id} a été trouvé.", id);
                return Ok(_inquiryService.GetInquiryById(id));
            }
            catch (ArgumentException)
            {
                _logger.LogError("[ERREUR] Aucune enquête trouvée avec l'id {id}.", id);
                return NotFound();
            }
        }

        [HttpGet("inquiry/title/{title}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 404)]
        public IActionResult GetInquiryByTitle(string title)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] Enquête avec le titre {title} a été trouvé.", title);
                return Ok(_inquiryService.GetInquiryByTitle(title));
            }
            catch (ArgumentException)
            {
                _logger.LogError("[ERREUR] Aucune enquête trouvée avec le titre {title}.", title);
                return NotFound();
            }
        }
    }
}